﻿<# 
.SYNOPSIS 
	Clones an existing Lync 2013 policy into a new policy with all existing settings.

.DESCRIPTION 
	Clones an existing Lync 2013 policy into a new policy with all existing settings. The new policy can then be further customized if needed.

.NOTES 
	Version							: 1.2 - See changelog at http://www.ehloworld.com/2302
	Wish list						: Better error trapping
	Rights Required			: CsAdministrator for some functions
	Sched Task Required	: No
	Lync Server Version	: 2013 (tested through CU4)
	PowerShell Version	: 3.0
	Author/Copyright		: © Pat Richard, Lync MVP - All Rights Reserved
	Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
	Dedicated Post			: http://www.ehloworld.com/2300
	Disclaimer 					: You running this script means you won't blame me if this breaks your stuff. This script is provided AS IS 
												without warranty of any kind. I disclaim all implied warranties including, without limitation, any implied 
												warranties of merchantability or of fitness for a particular purpose. The entire risk arising out of the use 
												or performance of the sample scripts and documentation remains with you. In no event shall I be liable for 
												any damages whatsoever (including, without limitation, damages for loss of business profits, business 
												interruption, loss of business information, or other pecuniary loss) arising out of the use of or inability 
												to use the script or documentation. 
	Acknowledgements 		: Brian Desmond (@brdesmond) for pointing out the InnerText method: http://msdn.microsoft.com/en-us/library/system.xml.xmlelement(v=vs.100).aspx
											: Using XML to copy policies: http://blogs.technet.com/b/nexthop/archive/2011/03/21/copypolicies.aspx
											: Changing XML data: http://powershell.com/cs/blogs/tobias/archive/2009/02/02/xml-part-2-write-add-and-change-xml-data.aspx
	Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
	Limitations					: Only works on Windows Server 2012. This is by design. There are no plans to support Windows Server 2008 R2.									
	Known issues				: None yet, but I'm sure you'll find some!

.LINK 
	http://www.ehloworld.com/2300

.EXAMPLE
	.\New-CsClonedPolicy.ps1 -PolicyType ConferencingPolicy -SourcePolicyName global -TargetPolicyName "New Policy"

	Description
	-----------
	Clones the Conferencing Policy called "global" into a new policy called "New Policy"

.INPUTS
	None. You cannot pipe objects to this script.
#>
#Requires -Version 3.0 
#Requires -Modules Lync

[CmdletBinding(SupportsShouldProcess = $True)]
param(
  # Folder location of the exported file while cloning the policy.
  [Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[ValidateNotNullOrEmpty()]
	[string]$ExportFolder = "$env:UserProfile\desktop",
  
  # Name of XML file used while the policy is being cloned.
  [Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[ValidateNotNullOrEmpty()]
	[string]$ExportFile = "ExportedPolicy.xml",
  
  # Defines the policy that will be copied.
  [Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[ValidateNotNullOrEmpty()]
	[string]$SourcePolicyName = "global",
  
	# Defines the name of the new policy. This policy cannot already exist.
  [Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[ValidateNotNullOrEmpty()]
	[string]$TargetPolicyName = "test",
  
  # Defines the type of policy being cloned. Valid values are "ArchivingPolicy", "ClientPolicy", "ClientVersionPolicy", "ConferencingPolicy", "ExternalAccessPolicy", "HostedVoicemailPolicy", "LocationPolicy", "MobilityPolicy", "NetworkInterSitePolicy", "PersistentChatPolicy", "PinPolicy", "PresencePolicy", "UserServicesPolicy", "VoicePolicy", "VoiceRoutingPolicy"
  [Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[ValidateNotNullOrEmpty()]
	[ValidateSet("ArchivingPolicy", "ClientPolicy", "ClientVersionPolicy", "ConferencingPolicy", "ExternalAccessPolicy", "HostedVoicemailPolicy", "LocationPolicy", "MobilityPolicy", "NetworkInterSitePolicy", "PersistentChatPolicy", "PinPolicy", "PresencePolicy", "UserServicesPolicy", "VoicePolicy", "VoiceRoutingPolicy")]
	[string]$PolicyType = "ConferencingPolicy",
	
	# Description for the new policy.
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
	[string]$Description = "Cloned from $SourcePolicyName. Created by New-CsClonedPolicy.ps1 (see http://www.ehloworld.com/2300)"
)

function Update-XMLData{
	[CmdletBinding(SupportsShouldProcess = $True)]
	param()
	if (!(Test-Path "$ExportFolder\$ExportFile")){
		Write-Output "Unable to export policy settings for `"$SourcePolicyName`" to $ExportFolder\$ExportFile"
		break
	}
	Write-Verbose "Updating XML data"
	$xml = New-Object XML
	$xml.Load("$ExportFolder\$ExportFile")
	($xml.Objs.Obj.Props.S | where {$_.N -eq "Identity"}).InnerText = $TargetPolicyName
	if ($xml.Objs.Obj.Props.S | where {$_.N -eq "Name"}){
		($xml.Objs.Obj.Props.S | where {$_.N -eq "Name"}).InnerText = $TargetPolicyName
	}
	Write-Verbose "Saving updated XML data"
	$xml.Save("$ExportFolder\$ExportFile")
} # end function Update-XMLData

[string] $get = "Get-Cs$PolicyType -Identity `"$SourcePolicyName`" | Export-Clixml -Path `"$ExportFolder\$ExportFile`""
[string] $set = "Import-Clixml -Path `"$ExportFolder\$ExportFile`" | Set-Cs$PolicyType"
if ($PolicyType -ne "UserServicesPolicy"){$set += " -Description `"$Description`""}
Write-Verbose "Exporting data"
Invoke-Expression $get
Update-XMLData
Write-Verbose "Importing updated XML data into new policy `"$TargetPolicyName`""
Invoke-Expression $set
Remove-Item "$ExportFolder\$ExportFile"

# SIG # Begin signature block
# MIIQBQYJKoZIhvcNAQcCoIIP9jCCD/ICAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUJu4A6pI5UHGxHskNxHxaDl8Q
# xzyggg1KMIIGnzCCBYegAwIBAgIQCSu3nvOdPAJrDHL+XciEEzANBgkqhkiG9w0B
# AQUFADBvMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMS4wLAYDVQQDEyVEaWdpQ2VydCBBc3N1cmVk
# IElEIENvZGUgU2lnbmluZyBDQS0xMB4XDTEzMDgyODAwMDAwMFoXDTE0MDkwNDEy
# MDAwMFowazELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAk1JMRkwFwYDVQQHExBTdGVy
# bGluZyBIZWlnaHRzMRkwFwYDVQQKExBJbm5lcnZhdGlvbiwgTExDMRkwFwYDVQQD
# ExBJbm5lcnZhdGlvbiwgTExDMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
# AQEA8dXC+TSUZsMisaialKJLat7AnmOF0HE+U49duaBJp/N4IXhjUZM9sXRkR/X0
# KAbNauNpNrs3TLKNpO7/RUinzrzsg8IhtBVXSrK7c0HDqJ7vG181it2qQO1yB9uD
# UV8frjprnNcR1w0qW9wFqclKlkyOxgiCANVoT6qXO9IxPZT28h+tMfBCsU7sZALs
# 3kPRMCQ1ZywngVSqEQ9rBYV3jW4EgvPEOmuhtnwgVB1Alr2V1dVAtbNxw1A7fQ8w
# J6EQSNAPVJQq7304QKKgGaTXtA2TwAjO5+m9ZltD/s0wXWYs0V1r8pOJOo1r17l4
# z91OWLPP+yfxvlnm5F1IXZxCbQIDAQABo4IDOTCCAzUwHwYDVR0jBBgwFoAUe2jO
# KarAF75JeuHlP9an90WPNTIwHQYDVR0OBBYEFB9N9j4QNDPcSscWMAVDKCOcXS/G
# MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBzBgNVHR8EbDBq
# MDOgMaAvhi1odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vYXNzdXJlZC1jcy0yMDEx
# YS5jcmwwM6AxoC+GLWh0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9hc3N1cmVkLWNz
# LTIwMTFhLmNybDCCAcQGA1UdIASCAbswggG3MIIBswYJYIZIAYb9bAMBMIIBpDA6
# BggrBgEFBQcCARYuaHR0cDovL3d3dy5kaWdpY2VydC5jb20vc3NsLWNwcy1yZXBv
# c2l0b3J5Lmh0bTCCAWQGCCsGAQUFBwICMIIBVh6CAVIAQQBuAHkAIAB1AHMAZQAg
# AG8AZgAgAHQAaABpAHMAIABDAGUAcgB0AGkAZgBpAGMAYQB0AGUAIABjAG8AbgBz
# AHQAaQB0AHUAdABlAHMAIABhAGMAYwBlAHAAdABhAG4AYwBlACAAbwBmACAAdABo
# AGUAIABEAGkAZwBpAEMAZQByAHQAIABDAFAALwBDAFAAUwAgAGEAbgBkACAAdABo
# AGUAIABSAGUAbAB5AGkAbgBnACAAUABhAHIAdAB5ACAAQQBnAHIAZQBlAG0AZQBu
# AHQAIAB3AGgAaQBjAGgAIABsAGkAbQBpAHQAIABsAGkAYQBiAGkAbABpAHQAeQAg
# AGEAbgBkACAAYQByAGUAIABpAG4AYwBvAHIAcABvAHIAYQB0AGUAZAAgAGgAZQBy
# AGUAaQBuACAAYgB5ACAAcgBlAGYAZQByAGUAbgBjAGUALjCBggYIKwYBBQUHAQEE
# djB0MCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wTAYIKwYB
# BQUHMAKGQGh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3Vy
# ZWRJRENvZGVTaWduaW5nQ0EtMS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0B
# AQUFAAOCAQEABTcbxhPxSJP7AKrF23wXbgelEGzjpewmm0BnHO9/V+9K4UE/3i5Z
# e3bX7OvUb/55zCHT4pvZl8/i8tuvFnrWZZcjGljT3W9SIVOuKNdlItXC8dvP67Hz
# OMZvSbdI5MU6sjd3vyYKznTMHG1Mz/x+BMh6rEnGsIaY4r8r4fkNfJTyTkww8AWh
# I16Pno3dWgKhfploPODtMxczJKA15VDFhQqM3oHHMojasXo5uGb5jlGiDFkZ3mDa
# /+gLUsJUjAzF27ZIy/jRgp0eMkqlgdxGa+gZUK73Ue6aTP2LW9SL/1fLYl8QkwRs
# MWG1faGdureT3Oh0EHH6UhREB/SxLJ+xLTCCBqMwggWLoAMCAQICEA+oSQYV1wCg
# viF2/cXsbb0wDQYJKoZIhvcNAQEFBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoT
# DERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UE
# AxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENBMB4XDTExMDIxMTEyMDAwMFoX
# DTI2MDIxMDEyMDAwMFowbzELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNl
# cnQgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EtMTCCASIwDQYJKoZIhvcNAQEB
# BQADggEPADCCAQoCggEBAJx8+aCPCsqJS1OaPOwZIn8My/dIRNA/Im6aT/rO38bT
# JJH/qFKT53L48UaGlMWrF/R4f8t6vpAmHHxTL+WD57tqBSjMoBcRSxgg87e98tzL
# uIZARR9P+TmY0zvrb2mkXAEusWbpprjcBt6ujWL+RCeCqQPD/uYmC5NJceU4bU7+
# gFxnd7XVb2ZklGu7iElo2NH0fiHB5sUeyeCWuAmV+UuerswxvWpaQqfEBUd9YCvZ
# oV29+1aT7xv8cvnfPjL93SosMkbaXmO80LjLTBA1/FBfrENEfP6ERFC0jCo9dAz0
# eotyS+BWtRO2Y+k/Tkkj5wYW8CWrAfgoQebH1GQ7XasCAwEAAaOCA0MwggM/MA4G
# A1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDAzCCAcMGA1UdIASCAbow
# ggG2MIIBsgYIYIZIAYb9bAMwggGkMDoGCCsGAQUFBwIBFi5odHRwOi8vd3d3LmRp
# Z2ljZXJ0LmNvbS9zc2wtY3BzLXJlcG9zaXRvcnkuaHRtMIIBZAYIKwYBBQUHAgIw
# ggFWHoIBUgBBAG4AeQAgAHUAcwBlACAAbwBmACAAdABoAGkAcwAgAEMAZQByAHQA
# aQBmAGkAYwBhAHQAZQAgAGMAbwBuAHMAdABpAHQAdQB0AGUAcwAgAGEAYwBjAGUA
# cAB0AGEAbgBjAGUAIABvAGYAIAB0AGgAZQAgAEQAaQBnAGkAQwBlAHIAdAAgAEMA
# UAAvAEMAUABTACAAYQBuAGQAIAB0AGgAZQAgAFIAZQBsAHkAaQBuAGcAIABQAGEA
# cgB0AHkAIABBAGcAcgBlAGUAbQBlAG4AdAAgAHcAaABpAGMAaAAgAGwAaQBtAGkA
# dAAgAGwAaQBhAGIAaQBsAGkAdAB5ACAAYQBuAGQAIABhAHIAZQAgAGkAbgBjAG8A
# cgBwAG8AcgBhAHQAZQBkACAAaABlAHIAZQBpAG4AIABiAHkAIAByAGUAZgBlAHIA
# ZQBuAGMAZQAuMBIGA1UdEwEB/wQIMAYBAf8CAQAweQYIKwYBBQUHAQEEbTBrMCQG
# CCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQwYIKwYBBQUHMAKG
# N2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJv
# b3RDQS5jcnQwgYEGA1UdHwR6MHgwOqA4oDaGNGh0dHA6Ly9jcmwzLmRpZ2ljZXJ0
# LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwOqA4oDaGNGh0dHA6Ly9j
# cmw0LmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwHQYD
# VR0OBBYEFHtozimqwBe+SXrh5T/Wp/dFjzUyMB8GA1UdIwQYMBaAFEXroq/0ksuC
# MS1Ri6enIZ3zbcgPMA0GCSqGSIb3DQEBBQUAA4IBAQB7ch1k/4jIOsG36eepxIe7
# 25SS15BZM/orh96oW4AlPxOPm4MbfEPE5ozfOT7DFeyw2jshJXskwXJduEeRgRNG
# +pw/alE43rQly/Cr38UoAVR5EEYk0TgPJqFhkE26vSjmP/HEqpv22jVTT8nyPdNs
# 3CPtqqBNZwnzOoA9PPs2TJDndqTd8jq/VjUvokxl6ODU2tHHyJFqLSNPNzsZlBjU
# 1ZwQPNWxHBn/j8hrm574rpyZlnjRzZxRFVtCJnJajQpKI5JA6IbeIsKTOtSbaKbf
# KX8GuTwOvZ/EhpyCR0JxMoYJmXIJeUudcWn1Qf9/OXdk8YSNvosesn1oo6WQsQz/
# MYICJTCCAiECAQEwgYMwbzELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNl
# cnQgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EtMQIQCSu3nvOdPAJrDHL+XciE
# EzAJBgUrDgMCGgUAoHgwGAYKKwYBBAGCNwIBDDEKMAigAoAAoQKAADAZBgkqhkiG
# 9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIB
# FTAjBgkqhkiG9w0BCQQxFgQUlA0th/aTKWqaNNFBARqZC15u7bIwDQYJKoZIhvcN
# AQEBBQAEggEALOGSOKGzpZQbywBYnWd5/9NPuqVB0tcFzIoqrrmrYcoRvkeHLHKz
# V755o1XvJNK28/AHvXTzVmo4VdtTMSTsISVtA+7dm6a1edV3pF5FViFT1eBaQJH6
# AkMj+6oj21oJjh22KChACfxDHbWaE2cdkhBOqQLZ9lJSi+WLqVPc8BiWKwWT3lpj
# NfFCtcYNw6MSPQ1UN0hyhaoMqWevgHbDTp748xA6d+Q6/IQQYbxvwuhwtJptZrO9
# PXgJ8XE+6uKexv+1WSmrkNtRagATuMW7xNeFEJ+yP2KvhafpEf+0/RCOQ10mw5Yy
# 1f2+X5X7c38s2F7n6Ti01wi6ddjBB4hb9w==
# SIG # End signature block
